
/*
 * auth_ldap.c  
 *
 * Authenticate against an existing LDAP tree. Authentication is done 
 * by searching for the users DN by a filter matching on its uid field
 * or whatever is used in the LDAP_FILTER - "%s" is replaced by the
 * username the client gives us. After finding the DN of the user the 
 * auth_ldap.c rebinds to the LDAP tree - now with password. The Server
 * handles all needed encryption which the password may be stored
 * in the TREE.
 *
 */
#include "config.h"

#ifdef AUTH_LDAP

#include <stdio.h>
#include <lber.h>
#include <ldap.h>
#include <sys/time.h>

#include "popd.h"

#ifndef LDAP_SEARCH_ROOT
#define LDAP_SEARCH_ROOT LDAP_BINDDN
#endif

int auth_ldap(struct pop_session *session) {
	LDAP		*ld;
	char		filter_buf[LDAP_FILTER_BUFSIZE+1],
			*dn,
			**attr_values;
	LDAPMessage	*res=0,
			*entry=0;
	struct timeval	tv;

	ld=ldap_open(LDAP_SERVER,LDAP_PORT);

	if (ld == 0) 
		return(POP_AUTH_ERROR);

	if (ldap_bind_s(ld, LDAP_BINDDN, NULL, LDAP_AUTH_SIMPLE) == LDAP_SUCCESS)  {

		tv.tv_sec = LDAP_TIMEOUT;
		tv.tv_usec = 0;

		snprintf(filter_buf, LDAP_FILTER_BUFSIZE, LDAP_FILTER, session->username);
		if (ldap_search_st(ld, LDAP_SEARCH_ROOT, LDAP_SCOPE_SUBTREE,
					filter_buf, 0, 0, &tv, &res) != -1) {

			if ((entry = ldap_first_entry(ld, res)) != 0) {
				dn=ldap_get_dn(ld,entry);
#ifdef LDAP_MAILSPOOL_ATTRIBUTE
				attr_values=ldap_get_values(ld, entry, LDAP_MAILSPOOL_ATTRIBUTE);
				if (attr_values) {
					strncpy(session->mboxpath,attr_values[0],MAX_POP_MBOXPATH);
					ldap_value_free(attr_values);
				}
#endif /* LDAP_MAILSPOOL_ATTRIBUTE */
			}
		}
	}

	if (res != 0) 
		ldap_msgfree(res);

	if (dn == NULL) {
		ldap_unbind(ld);
		return(POP_AUTH_ERROR);
	}

	/* Authenticate */
	if (ldap_bind_s(ld, dn, session->password, LDAP_AUTH_SIMPLE) != LDAP_SUCCESS) {
		ldap_unbind(ld);
		return(POP_AUTH_ERROR);	
	}

#ifdef LDAP_CHANGEUID
	session->uid=LDAP_CHANGEUID;
#endif
#ifdef LDAP_CHANGEGID
	session->gid=LDAP_CHANGEGID;
#endif

	ldap_unbind(ld);
	return(POP_AUTH_SUCCESS);
}

#endif /* AUTH_LDAP */
