
#include "config.h"

#ifdef MAILBOX_BSD

#include <stdio.h>
#include <errno.h>
#include <sys/stat.h>

#include "popd.h"

int mailbox_bsd_push_msg(struct st_message **messages, off_t offset,\
		int curmsg, int *max_msg) {

	if (curmsg >= (int ) max_msg) {
		*messages=(struct st_message *) realloc(*messages, (*max_msg+MESSAGES_INMEM * sizeof(struct st_message)));
		if (*messages == NULL)
			return(SCAN_FAILURE);

		max_msg+=MESSAGES_INMEM;
	}

	messages[0][curmsg].msg_no=curmsg;
	messages[0][curmsg].deleted=0;
	messages[0][curmsg].offset_start=offset;

}

int mailbox_bsd_scan(struct pop_session *session,int mboxfd) {
	struct	st_message	*messages;	
	struct	stat	stbuf;
	int		max_msgs=MESSAGES_INMEM,
			curmsg=-1,
			inheader=1;
	char		*matchstring=MAILSEPERATOR,
			*fp=matchstring,
			c,d='\n';
	int		i;
	unsigned long	uidl;
	off_t		oleft,
			totalsize=0;

	messages=(struct st_message *) malloc(MESSAGES_INMEM*sizeof(struct st_message));

	if (messages == NULL)
		return(SCAN_FAILURE);

	if (fstat(fileno((FILE *) mboxfd), &stbuf)) {
		return(SCAN_FAILURE);	
	}
	oleft=stbuf.st_size;

	/* Until no bytes left in the mailbox file */
	while(oleft >= 0) {
		/* Decrease left bytes */
		--oleft;
		/* Read a byte and compare it */
		if (*fp++ != (c=getc((FILE *) mboxfd))) {
			if (d == '\n' && c == '\n')
				inheader=0;
			d=c;
			fp=matchstring;
		} else {
			if (!*fp && d == '\n') {
				if (curmsg >= 0) {
					messages[curmsg].msg_size=(stbuf.st_size-oleft-sizeof(MAILSEPERATOR)+1)-messages[curmsg].offset_start;
					totalsize+=messages[curmsg].msg_size;
					messages[curmsg].uidl=uidl;
				}

				curmsg++;
				if (mailbox_bsd_push_msg(&messages,\
						stbuf.st_size-oleft-sizeof(MAILSEPERATOR)+1,\
						curmsg,&max_msgs) == SCAN_FAILURE) 
					return(SCAN_FAILURE);

				inheader=1; uidl=0;
			}
		}
		if (inheader == 1) {
			uidl=uidl*UIDL_MAGIC+(unsigned char )c;
		}
	}

	/* Store length of last message */
	if (curmsg >= 0) {
		messages[curmsg].msg_size=stbuf.st_size-messages[curmsg].offset_start;
		totalsize+=messages[curmsg].msg_size;
		messages[curmsg].uidl=uidl;
	}

	session->messages=messages;
	session->messages_count=curmsg;
	session->messages_highwater=curmsg;
	session->messages_size=totalsize;

	return(SCAN_SUCCESS);
}

int mailbox_bsd_open(struct pop_session *session) {
	int	mboxfd;
	int	openerrno;

	strncpy(session->mboxname,session->mboxpath,MAX_POP_MBOXPATH);

	/* append "/" if not there */
	if (session->mboxname[strlen(session->mboxname)] != '/') {
		strcat(session->mboxname,"/");
	}

	strncat(session->mboxname,session->username,\
			MAX_POP_MBOXPATH-strlen(session->mboxname));

	if (LOCK_LOCK(session) == LOCK_FAILURE) {
		return(MAILBOX_OPEN_FAILURE);
	}

	if (!(mboxfd=(int )fopen(session->mboxname,"r+"))) {
		if (errno == ENOENT) {
			/* We dont lock the mailboxfile - 
			 * we dont open it - there is no mailbox 
			 */
			session->messages_highwater=-1;
			session->messages_size=0;
			session->messages_count=-1;
			return(MAILBOX_OPEN_SUCCESS);
		}
		return(MAILBOX_OPEN_FAILURE);
	}

	if (mailbox_bsd_scan(session,mboxfd) == SCAN_FAILURE) {
		LOCK_UNLOCK(session);	
		fclose((FILE *) mboxfd);
		return(MAILBOX_OPEN_FAILURE);
	}

	session->mboxfd=mboxfd;
	session->mailboxopen=1;
		
	return(MAILBOX_OPEN_SUCCESS);
}


void mailbox_bsd_close(struct pop_session *session,int what) {
	off_t	rpos=0,
		wpos=0,
		left=0;
	int	i;
	char	*copybuffer;

	/* Just to be sure ... */
	if (session->mailboxopen == 0 || session->mboxfd == 0)
		return;

	/* Have we been terminated by command ? */
	if (what == DO_UPDATE) {

		copybuffer=(char *) malloc(MAILBOX_BSD_BLOCKSIZE);

		if (copybuffer != NULL) {
			for(i=0;i<=session->messages_highwater;i++) {

#ifdef MAILBOX_BSD_DEBUG
				syslog(LOG_DEBUG,"msg %i	rpos %9d offset %9d rpos-wpos %9d ",i,rpos,session->messages[i].offset_start,rpos-wpos);
#endif /* MAILBOX_BSD_DEBUG */

				if (session->messages[i].deleted == 1) {
#ifdef MAILBOX_BSD_DEBUG
					syslog(LOG_DEBUG,"\tDeleted - skipping\n");
#endif /* MAILBOX_BSD_DEBUG */
					rpos+=session->messages[i].msg_size;	
				} else if (wpos == rpos) {
#ifdef MAILBOX_BSD_DEBUG
					syslog(LOG_DEBUG,"\tintact and no move\n");
#endif /* MAILBOX_BSD_DEBUG */
					/* if we havent copied any mails skip forward
					 * for read AND write position
					 */
					wpos+=session->messages[i].msg_size;
					rpos+=session->messages[i].msg_size;
				} else {
#ifdef MAILBOX_BSD_DEBUG
					syslog(LOG_DEBUG,"\tintact and move\n");
#endif /* MAILBOX_BSD_DEBUG */
					/* we have different read & write pointers
					 * and a non-deleted mail - copy to the 
					 * front.
					 */
					left=session->messages[i].msg_size;

					/* Copy in full blocks if possible */
					while(left >= MAILBOX_BSD_BLOCKSIZE) {
#ifdef MAILBOX_BSD_DEBUG
						syslog(LOG_DEBUG,"\tLoop left %d rpos %9d wpos %9d rpos-wpos %9d\n",left,rpos,wpos,rpos-wpos);
#endif /* MAILBOX_BSD_DEBUG */

						lseek(fileno((FILE *) session->mboxfd),rpos,SEEK_SET);
						read(fileno((FILE *) session->mboxfd),copybuffer,MAILBOX_BSD_BLOCKSIZE);
						lseek(fileno((FILE *) session->mboxfd),wpos,SEEK_SET);
						write(fileno((FILE *) session->mboxfd),copybuffer,MAILBOX_BSD_BLOCKSIZE);

						left-=MAILBOX_BSD_BLOCKSIZE;
						rpos+=MAILBOX_BSD_BLOCKSIZE;
						wpos+=MAILBOX_BSD_BLOCKSIZE;
					}
					/* Normally we could not split mails into MAILBOX_BSD_BLOCKSIZE
					 * blocks without anything left - so we will copy it here ...
					 */
					if (left > 0) {
#ifdef MAILBOX_BSD_DEBUG
						syslog(LOG_DEBUG,"\tLeft: %d Rpos: %d Wpos: %d Mail: %d\n",left,rpos,wpos,i);
#endif /* MAILBOX_BSD_DEBUG */

						lseek(fileno((FILE *) session->mboxfd),rpos,SEEK_SET);
						read(fileno((FILE *) session->mboxfd),copybuffer,left);
						lseek(fileno((FILE *) session->mboxfd),wpos,SEEK_SET);
						write(fileno((FILE *) session->mboxfd),copybuffer,left);

						rpos+=left;
						wpos+=left;
					}
				}
			}
			/* Did we delete anything ? */
			if (wpos != rpos) {
				/* Not sure if this is needed */
				lseek(fileno((FILE *) session->mboxfd),wpos,SEEK_SET);
#ifdef MAILBOX_BSD_DEBUG
				syslog(LOG_DEBUG,"truncate rpos %ld wpos %ld\n",rpos,wpos);
#endif /* MAILBOX_BSD_DEBUG */
				/* truncate the mailfolder */
				ftruncate(fileno((FILE *) session->mboxfd),wpos);
			}
			/* free the buffer */
			free(copybuffer);
		} /* Copybuffer != NULL */
	} /* DO_UPDATE */	

	LOCK_UNLOCK(session);
	close((FILE *) session->mboxfd);

	session->mboxfd=session->mailboxopen=0;
}

int mailbox_bsd_retr(struct pop_session *session, int msg_no) {
	int	i;
	char	c,d=0x0;

	fseek((FILE *) session->mboxfd, session->messages[msg_no].offset_start, SEEK_SET);

	fprintf((FILE *) session->out, "+OK %d octets\r\n", 
				session->messages[msg_no].msg_size,
				session->messages[msg_no].offset_start);

	for(i=0;i<session->messages[msg_no].msg_size;i++) {
		c=fgetc((FILE *) session->mboxfd);
		
		/* quote . */
		if (c == '.' && d == '\n') {
			fputc('.',(FILE *) session->out);
		}

		fputc(c,(FILE *) session->out);
		d=c;
	}
	fprintf((FILE *) session->out, ".\n");

	return(CMD_POP_RESULT_SUCCESS);
}

int mailbox_bsd_dele(struct pop_session *session, int msg_no) {

	session->messages[msg_no].deleted=1;
	session->messages_size-=session->messages[msg_no].msg_size;
	session->messages_count--;

	fprintf((FILE *) session->out, "+OK Another one bites the dust\r\n");

	return(CMD_POP_RESULT_SUCCESS);
}

#endif /* MAILBOX_BSD */
