
#include "config.h"

#include <sys/socket.h>
#include <sys/wait.h>
#include <stdio.h>
#include <syslog.h>
#include <netinet/in.h>
#include <signal.h>

#ifdef USE_LIB_WRAP
#include <tcpd.h>
#endif

#include "popd.h"

int	debug,
	allow_severity=LOG_WARNING,
	deny_severity=LOG_WARNING;

void usage(void ) {
	printf("Usage:\n -d for stdin/out debugging\n");
}

void signal_child() {
        while (wait3((int *)NULL, WNOHANG, (struct rusage *)NULL) > 0);
}

int main(int argc, char **argv) {
	const char		*argptr;
	char			*daemonname=argv[0];
	struct sockaddr_in	peer;
	char			*hostname;
	int 			fdaccept,
				fdchild;
	struct sigaction	action;
	int			peerlen;

#ifdef USE_LIB_WRAP
	struct request_info	request;
#endif
	

	/* Parse command line arguments */
	while(argptr=*++argv) {
		if (*argptr++ == '-') {
			switch (*argptr++) {
				case 'd':debug=1;
					continue;
				default: {
					usage();
					return(EXIT_SUCCESS);
				}
			}
		} else {
			usage();
			return(EXIT_SUCCESS);
		}
	}

	openlog(NULL, LOG_PID, LOG_MAIL);

	/*
	 *
	 * Now we are getty busy ...
	 *
	 */ 
	if (debug) {
		startpop(stdin,stdout);
	} else {

		/* Prepare socket */
		fdaccept=socket(PF_INET, SOCK_STREAM, PROTOCOL_TCP);

		peer.sin_family=AF_INET;
		peer.sin_addr.s_addr=INADDR_ANY;
		peer.sin_port=htons(TCP_PORT_POP3);

		setsockopt(fdaccept, SOL_SOCKET, SO_REUSEADDR, &fdchild, sizeof(fdchild));

		if (bind(fdaccept,(struct sockaddr_in *) &peer, sizeof(peer))) {

			syslog(LOG_CRIT,"unable to bind to socket");
			fprintf(stderr,"unable to bind to socket\n");
			return(EXIT_FAILURE);

		}

		fclose(stdout);
		fclose(stdin);
		fclose(stderr);

		/* get to background */
		if (fork() > 0) 
			return(EXIT_SUCCESS);

		/* Get process group leadership */
		setsid();

		(void ) signal(SIGCHLD,signal_child);

		listen(fdaccept, NUM_BACKLOG);

		syslog(LOG_INFO,"waiting for connections");

		signal(SIGCHLD,&signal_child);

		while(1) {
			peerlen=sizeof(peer);
			fdchild=accept(fdaccept,(struct sockaddr *) &peer, &peerlen);

			/* We fork before checking libwrap etc
			 * because otherwise we would block until
			 * DNS and Identd succeeded or timed out
			 * which would be an easy DoS attack
			 */

			if (fdchild != -1) {
				syslog(LOG_INFO,"forking child");
				if (fork() == 0) {
					close(fdaccept);

#ifdef USE_LIB_WRAP
					/* initialize the tcp wrapper struct */
					request_init((struct request_info *) &request, \
						RQ_DAEMON, LIB_WRAP_DAEMON, RQ_FILE, fdchild, NULL);

					/* fill in the originating host */
					fromhost(&request);
				
					/* Check against hosts.{allow|deny} */
					if (!hosts_access(&request)) {
						syslog(LOG_ALERT,"tcp wrapper refused connection from %s",eval_client(&request));
						exit(0);
					}
#endif
					/* We succeeded tcp wrapper or didnt
					 * do anything ... Start popping ..
					 */
					startpop(fdopen(fdchild,"rb"),fdopen(fdchild,"wb"));

					/* Exit the child */
					exit(0);
				}
			}

			close(fdchild);

		}
			
	}
}
