
#include "config.h"

#include <sys/types.h>

struct st_message {
	int		msg_no;
	off_t		msg_size;
	off_t		offset_start;
	unsigned char	deleted;
	unsigned long	uidl;
};

struct pop_session {
	int			in,out;
	int			authenticated;
	int			pwcounter;
	int			uid,gid;
	char			username[MAX_POP_USERNAME];
	char			password[MAX_POP_PASSWORD];

	int			mailboxopen;
	
	int			messages_count;
	int			messages_highwater;
	off_t			messages_size;
	struct st_message	*messages;

	char			mboxpath[MAX_POP_MBOXPATH];

#ifdef MAILBOX_BSD
	char			mboxname[MAX_POP_MBOXPATH];
	int			mboxfd;
#endif

#ifdef LOCK_DOTLOCK
	char			mboxlock[MAX_POP_MBOXPATH+sizeof(LOCK_DOTLOCK_EXT)];
#endif /* LOCK_DOTLOCK */
};

typedef struct {
	int		state_call,
			state_ok;
	char		*cmdstring;
	int		args_min,
			args_max;
	int		(*cmd) (char **cmd, struct pop_session *session);
} type_cmd_table;

#define STATE_ALL	0
#define STATE_AUTH1	1
#define STATE_AUTH2	2
#define STATE_TRANS	3
#define STATE_KEEP	4

extern int cmd_pop_quit_noupdate(char **cmd,struct pop_session *session);
extern int cmd_pop_quit_update(char **cmd,struct pop_session *session);
extern int cmd_pop_retr(char **cmd,struct pop_session *session);
extern int cmd_pop_list(char **cmd,struct pop_session *session);
extern int cmd_pop_dele(char **cmd,struct pop_session *session);
extern int cmd_pop_user(char **cmd,struct pop_session *session);
extern int cmd_pop_pass(char **cmd,struct pop_session *session);
extern int cmd_pop_stat(char **cmd,struct pop_session *session);
extern int cmd_pop_rset(char **cmd,struct pop_session *session);
extern int cmd_pop_noop(char **cmd,struct pop_session *session);
extern int cmd_pop_uidl(char **cmd,struct pop_session *session);

extern char *getsysname(void );

#define CMD_POP_RESULT_SUCCESS	0
#define CMD_POP_RESULT_FAILURE	1
#define CMD_POP_RESULT_QUIT	2

#define POP_AUTH_FAILED		0
#define POP_AUTH_ERROR		1
#define POP_AUTH_SUCCESS	2

#define MAILBOX_OPEN_FAILURE	0
#define MAILBOX_OPEN_SUCCESS	1

#define LOCK_FAILURE		0
#define LOCK_SUCCESS		1

#define SCAN_FAILURE		0
#define SCAN_SUCCESS		1

#define EXIT_SUCCESS		0
#define EXIT_FAILURE		1

#define DONT_UPDATE		0
#define DO_UPDATE		0

#ifdef AUTH_PASSWD
#define AUTHENTICATE_USER	auth_passwd
extern int auth_passwd(struct pop_session *session);
#endif /* AUTH_PASSWD */

#ifdef AUTH_LDAP
#define AUTHENTICATE_USER	auth_ldap
extern int auth_passwd(struct pop_session *session);
#endif /*AUTH_LDAP*/
