#include "config.h"

#include <stdio.h>
#include <strings.h>
#include <malloc.h>
#include <syslog.h>

#include "popd.h"
#include "version.h"

extern char **readcmd(struct pop_session *session);
extern int cmd_unknown(char **cmd,struct pop_session *session);

static type_cmd_table cmd_table[] = {
	STATE_AUTH1,	STATE_AUTH2,	"user", 1, 1, &cmd_pop_user,
	STATE_AUTH1,	STATE_KEEP,	"quit", 0, 0, &cmd_pop_quit_noupdate,
	STATE_AUTH2,	STATE_TRANS,	"pass", 1, 1, &cmd_pop_pass,
	STATE_AUTH2,	STATE_KEEP,	"quit", 0, 0, &cmd_pop_quit_noupdate,
	STATE_TRANS,	STATE_KEEP,	"retr", 1, 1, &cmd_pop_retr,
	STATE_TRANS,	STATE_KEEP,	"list", 0, 1, &cmd_pop_list,
	STATE_TRANS,	STATE_KEEP,	"stat", 0, 1, &cmd_pop_stat,
	STATE_TRANS,	STATE_KEEP,	"dele", 1, 1, &cmd_pop_dele,
	STATE_TRANS,	STATE_KEEP,	"rset", 0, 0, &cmd_pop_rset,
	STATE_TRANS,	STATE_KEEP,	"noop", 0, 0, &cmd_pop_noop,
	STATE_TRANS,	STATE_KEEP,	"uidl", 0, 1, &cmd_pop_uidl,
	STATE_TRANS,	STATE_KEEP,	"quit", 0, 0, &cmd_pop_quit_update,
	STATE_ALL,	STATE_KEEP,	NULL, 	0, 0, &cmd_unknown
};

int cmd_unknown(char **cmd, struct pop_session *session) {
#ifdef LOG_UNKNOWN
	syslog(LOG_INFO,"unknown command %s",cmd[0]);
#endif
	fprintf((FILE *) session->out,"-ERR Unknown command %s\r\n",cmd[0]);
	return(CMD_POP_RESULT_SUCCESS);
}

void startpop(int in, int out) {
	char 			**cmd,
				*sysname;

	int			i,
				j,
				result,
				state=STATE_AUTH1;

	struct	pop_session	*session;

	/* Alloc a session structure */
	session=malloc(sizeof(struct pop_session));
	if (session == NULL) 
		return;

	memset(session,0x0,sizeof(struct pop_session));	

	/* Basic initialization */
	session->in=in;
	session->out=out;

	/* Set it now to allow the authentication modules to set the path */
	strncpy(session->mboxpath,MAILBOXPATH,MAX_POP_MBOXPATH);

	/* get systems name */
	sysname=getsysname();

	/* Send greeting */
	fprintf((FILE *) session->out, "+OK %s POP3 server (mafpop %s)\r\n",
				sysname,
				POPD_VERSION);

	fflush((FILE *) session->out);

	while(session != NULL) {

		cmd=readcmd(session);

		if (cmd == NULL)
			break;

		/* Count the command - We also count
		 * command itself and the NULL pointer
		 * so substract 2
		 */
		j=0;
		while((char * )cmd[j++] != NULL);
		j-=2;

		for(i=0;;i++) {

			/* Did we run off the end of the table ? */
			if (cmd_table[i].cmdstring == NULL) {
				fprintf((FILE *) session->out, "-ERR illegal command\r\n");
				fflush((FILE *) session->out);
				break;
			}

			/* Correct command and state ? */
			if (!strcmp(cmd[0],cmd_table[i].cmdstring) && \
				(cmd_table[i].state_call == STATE_ALL || \
				cmd_table[i].state_call == state)) {

				/* Number of arguments correct ? */
				if (cmd_table[i].args_min > j || cmd_table[i].args_max < j) 
					fprintf((FILE *) session->out, "-ERR too %s arguments\r\n",
								(cmd_table[i].args_min > j) ? "less" : "many");
				else {
					result=cmd_table[i].cmd(cmd,session);

					fflush((FILE *) session->out);

					if (result == CMD_POP_RESULT_SUCCESS && \
							cmd_table[i].state_ok != STATE_KEEP) 
						state=cmd_table[i].state_ok;

					if (result == CMD_POP_RESULT_QUIT) {
						free(session);
						session=NULL;
					}
				}
				break;
			}
		}
	}

	/* and exit */
	syslog(LOG_INFO,"client exiting");
}
